#include <stdio.h>
#include <stdlib.h>
#include <string.h>

  /***** (a) Definicija tipa turnir. *****/

    /* 1. nacin:
       prvo definiramo tip strukture igrac_t za jednog igraca,
       a onda turnir ima polje takvih struktura. */

/*
typedef struct {
    char ime_igraca[21];
    int bodovi;
} igrac_t;

typedef struct {
    char ime_turnira[21];
    igrac_t igrac[128];
} turnir;
*/

    /* 2. nacin:
       spojimo sve u jedan tip za turnir, tako da tip strukture za jednog
       igraca definiramo kao tip za clan strukture, a onda, za clan
       navedemo polje takvih struktura. */
    /* Ime strukture _igrac za jednog igraca nam treba samo kod
       zamjene igraca u poredaj. Inace se smije ispustiti. */

typedef struct {
    char ime_turnira[21];
    struct _igrac {
        char ime_igraca[21];
        int bodovi;
    } igrac[128];
} turnir;

  /***** (b) Funkcija poredaj *****/

turnir poredaj(turnir t)
{
    int i, j, i_min;

        /* Za 1. nacin definicije tipa, imamo posebni tip za igraca. */
/*
    igrac_t tmp;
*/
        /* Za 2. nacin, treba navesti ime strukture kao tip za igraca. */
    struct _igrac tmp;

        /* Sortiranje trazenjem ektrema. */

    for (i = 0; i < 127; ++i) {
        i_min = i;
        for (j = i + 1; j < 128; ++j)
            if (t.igrac[j].bodovi < t.igrac[i_min].bodovi)
                i_min = j;

            /* Zamjena cijelih igraca. */

        if (i_min > i) {
            tmp = t.igrac[i];
            t.igrac[i] = t.igrac[i_min];
            t.igrac[i_min] = tmp;
        }
    }
    return t;
}

  /***** (c) Funkcija bodovi *****/

int bodovi(char *ime_igraca, char *ime_datoteke)
{
    FILE *fp;
    turnir t;
    int i, suma = 0;

        /* Otvori datoteku s provjerom. */

    if ((fp = fopen(ime_datoteke, "rb")) == NULL) {
        fprintf(stderr, "Ne mogu otvoriti datoteku %s!\n", ime_datoteke);
        exit(EXIT_FAILURE);  // ili samo return;
    }

        /* Prolaz kroz cijelu datoteku i provjera svakog turnira. */

    while (fread(&t, sizeof(turnir), 1, fp) == 1) {
        for (i = 0; i < 128; ++i) {
            if (strcmp(ime_igraca, t.igrac[i].ime_igraca) == 0) {
                suma += t.igrac[i].bodovi;

                    /* Uz pretpostavku da je igrac naveden samo jednom,
                       onda moze i donji break - za prekid for petlje. */
                break;
            }
        }
    }

    fclose(fp);
    return suma;
}

  /***** (d) Funkcija uredi *****/

void uredi(FILE *fp)
{
    turnir t;

        /* Postavi se na pocetak datoteke, za slucaj da nismo tamo. */

    rewind(fp);  // moze i fseek(fp, 0L, SEEK_SET);

        /* Prolaz kroz cijelu datoteku i uredjenje svakog turnira. */

    while (fread(&t, sizeof(turnir), 1, fp) == 1) {
        t = poredaj(t);

            /* Pomak unatrag za jedan turnir i pisanje. */

        fseek(fp, -sizeof(turnir), SEEK_CUR);
        fwrite(&t, sizeof(turnir), 1, fp);
    }

    return;
}

  /***** Test program *****/

int main(int argc, char *argv[])
{
    FILE *fp;

    if (argc != 3) {
        fprintf(stderr, "Greska, nema imena datoteke i igraca!\n");
        exit(EXIT_FAILURE);
    }

        /* Uzmimo da je argv[1] = ime datoteke, argv[2] = ime igraca. */

    printf("Broj bodova igraca %s na turnirima u datoteci %s je %d\n",
           argv[2], argv[1], bodovi(argv[2], argv[1]));

        /* Otvori datoteku za citanje i pisanje, prije poziva uredi. */

    if ((fp = fopen(argv[1], "r+b")) == NULL) {
        fprintf(stderr, "Ne mogu otvoriti datoteku %s!\n", argv[1]);
        exit(EXIT_FAILURE);
    }

    uredi(fp);
    fclose(fp);

    return EXIT_SUCCESS;
}
